#!/usr/bin/env python
# coding: utf-8

from typing import List

from evidently.tabs.base_tab import Tab
from evidently.widgets.big_drift_table_widget import BigDriftTableWidget
from evidently.widgets.widget import Widget


class DriftTab(Tab):
    def _get_widgets(self) -> List[Widget]:
        return [
            BigDriftTableWidget("Data Drift Table")
        ]


#!/usr/bin/env python
# coding: utf-8

import dataclasses
import json
import os
import uuid
import base64
from dataclasses import asdict
from typing import List, Dict, Type

import pandas
import typing

import evidently
from evidently.model.dashboard import DashboardInfo
from evidently.tabs.base_tab import Tab


@dataclasses.dataclass()
class TemplateParams:
    dashboard_id: str
    dashboard_info: DashboardInfo
    additional_graphs: Dict


def __dashboard_info_to_json(di: DashboardInfo):
    return json.dumps(asdict(di))


def inline_template(params: TemplateParams):
    return f"""
<link rel="stylesheet" href="https://fonts.googleapis.com/icon?family=Material+Icons"/>
<style>
.reset-this-parent {{
  all: initial;
  * {{
    all: unset;
  }}
}}
</style>
<script>
    var {params.dashboard_id} = {__dashboard_info_to_json(params.dashboard_info)};
    var additional_graphs_{params.dashboard_id} = {json.dumps(params.additional_graphs)};
    console.log(additional_graphs_{params.dashboard_id});
</script>
<script>
$(document).ready(function () {{
    requirejs(["evidently"], function(ev) {{
        drawDashboard({params.dashboard_id},
        new Map(Object.entries(additional_graphs_{params.dashboard_id})),
        "root_{params.dashboard_id}");
    }},
    function(err) {{
        $("#root_{params.dashboard_id}").innerHTML = "Failed to load";
    }})
}});
</script>
<div class="reset-this-parent" id="root_{params.dashboard_id}">Loading...</div>

"""


def file_html_template(params: TemplateParams):
    return f"""
<html>
<head>
<style>
/* fallback */
@font-face {{
  font-family: 'Material Icons';
  font-style: normal;
  font-weight: 400;
  src: url(data:font/ttf;base64,{__load_font()}) format('woff2');
}}

.material-icons {{
  font-family: 'Material Icons';
  font-weight: normal;
  font-style: normal;
  font-size: 24px;
  line-height: 1;
  letter-spacing: normal;
  text-transform: none;
  display: inline-block;
  white-space: nowrap;
  word-wrap: normal;
  direction: ltr;
  text-rendering: optimizeLegibility;
  -webkit-font-smoothing: antialiased;
}}
</style>
<script>
    var {params.dashboard_id} = {__dashboard_info_to_json(params.dashboard_info)};
    var additional_graphs_{params.dashboard_id} = {json.dumps(params.additional_graphs)};
    console.log(additional_graphs_{params.dashboard_id});
</script>
<script>{__load_js()}</script>
</head>
<body>
<div id="root_{params.dashboard_id}">Failed to load</div>
<script>
drawDashboard({params.dashboard_id},
    new Map(Object.entries(additional_graphs_{params.dashboard_id})),
    "root_{params.dashboard_id}"
);
</script>
</body>
"""


__base_path = evidently.__path__[0]
__static_path = os.path.join(__base_path, "nbextension", "static")


def __load_js():
    return open(os.path.join(__static_path, "index.js")).read()


def __load_font():
    return base64.b64encode(
        open(os.path.join(__static_path, "material-ui-icons.woff2"), 'rb').read()).decode()


class Dashboard:
    name: str

    def __init__(self,
                 reference_data: pandas.DataFrame,
                 production_data: pandas.DataFrame,
                 tabs: List[Type[Tab]],
                 column_mapping = None):
        self.reference_data = reference_data
        self.production_data = production_data
        self.tabsData = [t() for t in tabs]
        for tab in self.tabsData:
            tab.calculate(reference_data, production_data, column_mapping)

    def __render(self, template: typing.Callable[[TemplateParams], str]):
        dashboard_id = "evidently_dashboard_" + str(uuid.uuid4()).replace("-", "")
        tab_widgets = [t.info() for t in self.tabsData]
        di = DashboardInfo(dashboard_id, [item for tab in tab_widgets for item in tab])
        additional_graphs = {}
        for widget in [item for tab in tab_widgets for item in tab]:
            for graph in widget.additionalGraphs:
                additional_graphs[graph.id] = graph.params

        return template(TemplateParams(dashboard_id, di, additional_graphs))

    def show(self):
        from IPython.display import HTML
        return HTML(self.__render(inline_template))

    def save(self, filename):
        parent_dir = os.path.dirname(filename)
        if parent_dir and not os.path.exists(parent_dir):
            os.makedirs(parent_dir, exist_ok=True)
        f = open(filename, 'w')
        f.write(self.__render(file_html_template))



#!/usr/bin/env python
# coding: utf-8

import abc
from typing import List, Dict

import pandas

from evidently.model.widget import BaseWidgetInfo
from evidently.widgets.widget import Widget


class Tab:
    widgets: List[Widget]

    def __init__(self):
        self.widgets = []

    def calculate(self, reference_data: pandas.DataFrame,
                  production_data: pandas.DataFrame,
                  column_mapping: Dict):
        self.widgets = self._get_widgets()
        for widget in self.widgets:
            widget.calculate(reference_data, production_data, column_mapping)

    def info(self) -> List[BaseWidgetInfo]:
        return [w.get_info() for w in self.widgets]

    @abc.abstractmethod
    def _get_widgets(self) -> List[Widget]:
        raise NotImplemented()



#!/usr/bin/env python
# coding: utf-8

import abc

import pandas

from evidently.model.widget import BaseWidgetInfo
from evidently.widgets.widget import Widget


class RawWidget(Widget):
    @abc.abstractmethod
    def calculate(self, reference_data: pandas.DataFrame, production_data: pandas.DataFrame, column_mapping):
        raise NotImplemented()

    def __init__(self, wi: BaseWidgetInfo):
        super().__init__()
        self.wi = wi

    def get_info(self) -> BaseWidgetInfo:
        return self.wi



#!/usr/bin/env python
# coding: utf-8

import json
import pandas as pd
from pandas.api.types import is_numeric_dtype
import numpy as np

from scipy.stats import ks_2samp, chisquare
import matplotlib.pyplot as plt
import plotly.graph_objs as go

from evidently.model.widget import BaseWidgetInfo, AlertStats, AdditionalGraphInfo
from evidently.widgets.widget import Widget

red = "#ed0400"
grey = "#4d4d4d"


class BigDriftTableWidget(Widget):
    def __init__(self, title: str):
        super().__init__()
        self.title = title

    def get_info(self) -> BaseWidgetInfo:
        if self.wi:
            return self.wi
        raise ValueError("no widget info provided")

    def calculate(self, reference_data: pd.DataFrame, production_data: pd.DataFrame, column_mapping): 
        if column_mapping:
            date_column = column_mapping.get('datetime')
            id_column = column_mapping.get('id')
            target_column = column_mapping.get('target')
            prediction_column = column_mapping.get('prediction')
            num_feature_names = column_mapping.get('numerical_features')
            if num_feature_names is None:
                num_feature_names = []
            else:
                num_feature_names = [name for name in num_feature_names if is_numeric_dtype(reference_data[name])] 

            cat_feature_names = column_mapping.get('categorical_features')
            if cat_feature_names is None:
                cat_feature_names = []
            else:
                cat_feature_names = [name for name in cat_feature_names if is_numeric_dtype(reference_data[name])] 
        
        else:
            date_column = 'datetime' if 'datetime' in reference_data.columns else None
            id_column = None
            target_column = 'target' if 'target' in reference_data.columns else None
            prediction_column = 'prediction' if 'prediction' in reference_data.columns else None

            utility_columns = [date_column, id_column, target_column, prediction_column]

            num_feature_names = list(set(reference_data.select_dtypes([np.number]).columns) - set(utility_columns))
            cat_feature_names = list(set(reference_data.select_dtypes([np.object]).columns) - set(utility_columns))

        #set params data
        params_data = []
        plt.ioff()
        for feature_name in num_feature_names:# + cat_feature_names: #feature_names:
            prod_small_hist = plt.hist(production_data[feature_name], density = True)
            ref_small_hist = plt.hist(reference_data[feature_name], density = True)

            feature_type = 'num'

            p_value = ks_2samp(reference_data[feature_name], production_data[feature_name])[1]

            distr_sim_test = "Detected" if p_value < 0.05 else "Not Detected"

            params_data.append(
                {
                    "details": {
                            "parts": [
                                {
                                    "title": "Data drift",
                                    "id": feature_name + "_drift"
                                },
                                {
                                    "title": "Data distribution",
                                    "id": feature_name + "_distr"
                                }
                            ],
                            "insights": []
                        },
                        "f1": feature_name,
                        "f6": feature_type,
                        "f3": {
                            "x": list(ref_small_hist[1]),
                            "y": list(ref_small_hist[0])
                        },
                        "f4": {
                            "x": list(prod_small_hist[1]),
                            "y": list(prod_small_hist[0])
                        },
                        "f2": distr_sim_test,
                        "f5": round(p_value, 6) 
                }
                )

        for feature_name in cat_feature_names: #feature_names:
            prod_small_hist = plt.hist(production_data[feature_name], density = True)
            ref_small_hist = plt.hist(reference_data[feature_name], density = True)

            feature_type = 'cat'

            p_value = ks_2samp(reference_data[feature_name], production_data[feature_name])[1]
            #CHI2 to be implemented for cases with different categories
            #p_value = chisquare(reference_data[feature_name], production_data[feature_name])[1]

            distr_sim_test = "Detected" if p_value < 0.05 else "Not Detected"

            params_data.append(
                {
                    "details": {
                            "parts": [
                                {
                                    "title": "Data drift",
                                    "id": feature_name + "_drift"
                                },
                                {
                                    "title": "Data distribution",
                                    "id": feature_name + "_distr"
                                }
                            ],
                            "insights": []
                        },
                        "f1": feature_name,
                        "f6": feature_type,
                        "f3": {
                            "x": list(ref_small_hist[1]),
                            "y": list(ref_small_hist[0])
                        },
                        "f4": {
                            "x": list(prod_small_hist[1]),
                            "y": list(prod_small_hist[0])
                        },
                        "f2": distr_sim_test,
                        "f5": round(p_value, 6) 
                }
                )


        #set additionalGraphs
        additional_graphs_data = []
        for feature_name in num_feature_names + cat_feature_names: #feature_names:

            #plot distributions
            fig = go.Figure()
            fig.add_trace(go.Histogram(x=reference_data[feature_name], 
                 marker_color=grey, opacity=0.6, nbinsx=10,  name = 'Reference', histnorm='probability'))

            fig.add_trace(go.Histogram(x=production_data[feature_name],
                 marker_color=red, opacity=0.6,nbinsx=10, name = 'Production', histnorm='probability'))

            fig.update_layout(
                legend = dict(
                orientation="h",
                yanchor="bottom",
                y=1.02,
                xanchor="right",
                x=1
                ),
                xaxis_title = feature_name,
                yaxis_title = "Share"
            )
    
            distr_figure = json.loads(fig.to_json())
        
            #plot drift
            reference_mean = np.mean(reference_data[feature_name])
            reference_std = np.std(reference_data[feature_name], ddof = 1) 
            x_title = "Timestamp" if date_column else "Index"

            fig = go.Figure()

            fig.add_trace(go.Scatter(
                x = production_data[date_column] if date_column else production_data.index,
                y = production_data[feature_name],
                mode = 'markers',
                name = 'Production',
                marker = dict(
                    size = 6,
                    color = grey
                )
            ))


            fig.update_layout(
                xaxis_title = x_title,
                yaxis_title = feature_name,
                showlegend = True,
                legend = dict(
                orientation = "h",
                yanchor = "bottom",
                y=1.02,
                xanchor = "right",
                x = 1
                ),
                shapes = [
                    dict(
                        type = "rect",
                        # x-reference is assigned to the x-values
                        xref = "paper",
                        # y-reference is assigned to the plot paper [0,1]
                        yref = "y",
                        x0 = 0, 
                        y0 = reference_mean - reference_std, 
                        x1 = 1, 
                        y1 = reference_mean + reference_std, 
                        fillcolor = "LightGreen",
                        opacity = 0.5,
                        layer = "below",
                        line_width = 0,
                    ),
                    dict(
                        type="line",
                        name = 'Reference',
                        xref = "paper",
                        yref = "y",
                        x0 = 0, #min(testset_agg_by_date.index),
                        y0 = reference_mean,
                        x1 = 1, #max(testset_agg_by_date.index),
                        y1 = reference_mean,
                        line = dict(
                            color = "Green",
                            width = 3
                            )
                    ),
                ]  
            )

            drift_figure = json.loads(fig.to_json())

            #add distributions data
            additional_graphs_data.append(
                AdditionalGraphInfo(
                    feature_name + '_distr',
                    {
                        "data" : distr_figure['data'],
                        "layout" : distr_figure['layout']
                    }
                )
            )

            #add drift data
            additional_graphs_data.append(
                AdditionalGraphInfo(
                    feature_name + '_drift',
                    {
                        "data" : drift_figure['data'],
                        "layout" : drift_figure['layout']
                    }
                )
            )

        self.wi = BaseWidgetInfo(
            title=self.title,
            type="big_table",
            details="",
            alertStats=AlertStats(),
            alerts=[],
            alertsPosition="row",
            insights=[],
            size=2,
            params={
                "rowsPerPage" : min(len(num_feature_names) + len(cat_feature_names), 10),
                "columns": [
                    {
                        "title": "Feature",
                        "field": "f1"
                    },
                    {
                        "title": "Type",
                        "field": "f6"
                    },
                    {
                        "title": "Reference Distribution",
                        "field": "f3",
                        "type": "histogram",
                        "options": {
                            "xField": "x",
                            "yField": "y"
                        }
                    },
                    {
                        "title": "Production Distribution",
                        "field": "f4",
                        "type": "histogram",
                        "options": {
                            "xField": "x",
                            "yField": "y"
                        }
                    },
                    {
                        "title": "Data drift",
                        "field": "f2"
                    },
                    {
                        "title": "P-Value for Similarity Test",
                        "field": "f5",
                        "sort" : "asc"
                    }
                ],
                "data": params_data
            },

            additionalGraphs = additional_graphs_data
        )



#!/usr/bin/env python
# coding: utf-8

import abc

import pandas

from evidently.model.widget import BaseWidgetInfo


class Widget:
    def __init__(self):
        pass

    @abc.abstractmethod
    def calculate(self, reference_data: pandas.DataFrame,
                  production_data: pandas.DataFrame, column_mapping):
        raise NotImplemented()

    @abc.abstractmethod
    def get_info(self) -> BaseWidgetInfo:
        raise NotImplemented()



#!/usr/bin/env python
# coding: utf-8

# Copyright (c) Jupyter Development Team.
# Distributed under the terms of the Modified BSD License.

from __future__ import print_function
from glob import glob
from os.path import join as pjoin

from setupbase import (
    create_cmdclass, ensure_targets,
    find_packages, combine_commands, ensure_python,
    get_version, HERE
)

from setuptools import setup

# The name of the project
name = 'evidently'

# Ensure a valid python version
ensure_python('>=3.4')

# Get our version
version = get_version(pjoin(name, '_version.py'))

nb_path = pjoin(HERE, name, 'nbextension', 'static')
lab_path = pjoin(HERE, name, 'labextension')

# Representative files that should exist after a successful build
jstargets = [
    pjoin(nb_path, 'index.js'),
    # pjoin(HERE, 'lib', 'plugin.js'),
]

package_data_spec = {
    name: [
        'nbextension/static/*.*js*',
        'nbextension/static/*.woff2',
        'labextension/*.tgz'
    ]
}

data_files_spec = [
    ('share/jupyter/nbextensions/evidently',
     nb_path, '*.js*'),
    ('share/jupyter/lab/extensions', lab_path, '*.tgz'),
    ('etc/jupyter/nbconfig/notebook.d', HERE, 'evidently.json')
]

cmdclass = create_cmdclass('jsdeps', package_data_spec=package_data_spec,
                           data_files_spec=data_files_spec)
cmdclass['jsdeps'] = combine_commands(
    # install_npm(HERE, build_cmd='build:all'),
    ensure_targets(jstargets),
)

setup_args = dict(
    name=name,
    description='Open-source tools to analyze, monitor, and debug machine learning model in production.',
    version=version,
    scripts=glob(pjoin('scripts', '*')),
    cmdclass=cmdclass,
    packages=find_packages(),
    author='Emeli Dral',
    author_email='emeli.dral@gmail.com',
    url='https://github.com/evidentlyai/evidently',
    license='Apache License 2.0',
    platforms="Linux, Mac OS X, Windows",
    keywords=[],
    classifiers=[],
    include_package_data=True,
    install_requires=[
        "dataclasses",
        "pandas",
        "numpy",
        "matplotlib",
        "plotly",
        "scipy",
    ],
    entry_points={
    },
)

if __name__ == '__main__':
    setup(**setup_args)



#!/usr/bin/env python
# coding: utf-8

from typing import Dict, Optional

import pandas

from evidently.model.widget import BaseWidgetInfo, AlertStats
from evidently.widgets.widget import Widget


class CounterWidget(Widget):
    wi: Optional[BaseWidgetInfo]

    def __init__(self, title: str):
        super().__init__()
        self.wi = None
        self.title = title

    def calculate(self, reference_data: pandas.DataFrame, production_data: pandas.DataFrame, _: Dict):
        self.wi = BaseWidgetInfo(
            type="counter",
            title=self.title,
            size=2,
            params={
                "counters": [
                    {
                        "value": "7 out of 12 features",
                        "label": "Data Drift Detected"
                    }
                ]
            },
            alerts=[],
            insights=[],
            details="",
            alertsPosition="row",
            alertStats=AlertStats(),
            additionalGraphs=[],
        )

    def get_info(self) -> BaseWidgetInfo:
        if self.wi:
            return self.wi
        raise ValueError("no widget info provided")



#!/usr/bin/env python
# coding: utf-8

from dataclasses import dataclass
from typing import List, Any


@dataclass
class AlertStats:
    pass


@dataclass
class Insight:
    pass


@dataclass
class Alert:
    pass


@dataclass
class AdditionalGraphInfo:
    id: str
    params: Any


@dataclass
class BaseWidgetInfo:
    type: str
    title: str
    size: int
    details: str
    params: Any
    alertsPosition: str
    alertStats: AlertStats
    alerts: List[Alert]
    insights: List[Insight]
    additionalGraphs: List[AdditionalGraphInfo]


